<?php

class Weblator_Charts {

	/**
	 * Plugin version, used for cache-busting of style and script file references.
	 *
	 * @since   1.0.0
	 *
	 * @var     string
	 */
	const VERSION = '1.2.5';

	/**
	 * Unique identifier for your plugin.
	 *
	 *
	 * The variable name is used as the text domain when internationalizing strings
	 * of text. Its value should match the Text Domain file header in the main
	 * plugin file.
	 *
	 * @since    1.0.0
	 *
	 * @var      string
	 */
	protected $plugin_slug = 'weblator-charts';

	/**
	 * Instance of this class.
	 *
	 * @since    1.0.0
	 *
	 * @var      object
	 */
	protected static $instance = null;


    /**
     * Shortcode.
     *
     * @since    1.2.3
     *
     * @var      string
     */
    protected $short_code = null;


	/**
	 * Initialize the plugin by setting localization and loading public scripts
	 * and styles.
	 *
	 * @since     1.0.0
	 */
	private function __construct() {

        add_action( 'wpmu_new_blog', array( $this, 'activate_new_site' ) );
		// Load public-facing style sheet and JavaScript.
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );


        add_shortcode("chart", array( $this, 'display_chart'));
        add_shortcode("weblator_chart", array( $this, 'display_chart'));

        add_action('wp_head', array($this, 'ajax_url') );

	}

	/**
	 * Return the plugin slug.
	 *
	 * @since    1.0.0
	 *
	 * @return    Plugin slug variable.
	 */
	public function get_plugin_slug() {
		return $this->plugin_slug;
	}


	/**
	 * Return an instance of this class.
	 *
	 * @since     1.0.0
	 *
	 * @return    object    A single instance of this class.
	 */
	public static function get_instance() {

		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}


    public static function activate( $network_wide ) {

        if ( function_exists( 'is_multisite' ) && is_multisite() ) {

            if ( $network_wide  ) {

                // Get all blog ids
                $blog_ids = self::get_blog_ids();

                foreach ( $blog_ids as $blog_id ) {

                    switch_to_blog( $blog_id );
                    self::single_activate();
                }

                restore_current_blog();

            } else {
                self::single_activate();
            }

        } else {
            self::single_activate();
        }

    }

    /**
     * Fired when the plugin is deactivated.
     *
     * @since    1.0.0
     *
     * @param    boolean    $network_wide
     */
    public static function deactivate( $network_wide ) {

        if ( function_exists( 'is_multisite' ) && is_multisite() ) {

            if ( $network_wide ) {

                // Get all blog ids
                $blog_ids = self::get_blog_ids();

                foreach ( $blog_ids as $blog_id ) {

                    switch_to_blog( $blog_id );
                    self::single_deactivate();

                }

                restore_current_blog();

            } else {
                self::single_deactivate();
            }

        } else {
            self::single_deactivate();
        }

    }

    /**
     * Fired when a new site is activated with a WPMU environment.
     *
     * @since    1.0.0
     *
     * @param    int    $blog_id    ID of the new blog.
     */
    public function activate_new_site( $blog_id ) {

        if ( 1 !== did_action( 'wpmu_new_blog' ) ) {
            return;
        }

        switch_to_blog( $blog_id );
        self::single_activate();
        restore_current_blog();

    }

    /**
     * Get all blog ids of blogs in the current network that are:
     * - not archived
     * - not spam
     * - not deleted
     *
     * @since    1.0.0
     *
     * @return   array|false    The blog ids, false if no matches.
     */
    private static function get_blog_ids() {

        global $wpdb;

        // get an array of blog ids
        $sql = "SELECT blog_id FROM $wpdb->blogs
			WHERE archived = '0' AND spam = '0'
			AND deleted = '0'";

        return $wpdb->get_col( $sql );

    }


    private static function single_activate() {

        global $wpdb;
        //Create Tables
        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_chart_options` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `chart_id` int(11) DEFAULT NULL,
              `option_name` varchar(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL,
              `option_value` varchar(255) DEFAULT NULL,
              `option_order` int(11) DEFAULT NULL,
              `option_colour` varchar(255) DEFAULT NULL,
              `option_deleted_date` timestamp NULL DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
        );

        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_chart_type` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `chart_type` varchar(50) DEFAULT NULL,
              `order` int(11) DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
        );

        $wpdb->query(
          "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_charts` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `chart_name` varchar(255) CHARACTER SET utf8 COLLATE utf8_unicode_ci DEFAULT NULL,
              `chart_type` int(11) DEFAULT NULL,
              `chart_is_live` tinyint(1) DEFAULT NULL,
              `chart_max_width` int(11) DEFAULT NULL,
              `chart_legend` tinyint(1) DEFAULT '0',
              `chart_legend_position` varchar(2) DEFAULT 'tl',
              `chart_legend_font_size` varchar(10) DEFAULT '12',
              `chart_legend_font_style` varchar(10) DEFAULT 'normal',
              `chart_legend_font_colour` varchar(50) DEFAULT 'rgb(102, 102, 102)',
              `created_date` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
              `deleted_date` timestamp NULL DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
        );

        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_charts_style_value` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `chart_id` int(11) DEFAULT NULL,
              `style_id` int(11) DEFAULT NULL,
              `style_value` varchar(250) DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
        );

        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_style_chart_link` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `chart_id` int(11) DEFAULT NULL,
              `style_id` int(11) DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
        );

        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_style_options` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `style_option` varchar(250) DEFAULT NULL,
              `style_key` varchar(25) DEFAULT NULL,
              `style_label` varchar(255) DEFAULT NULL,
              `style_description` text,
              `style_colorpicker` tinyint(1) DEFAULT NULL,
              `style_bool` tinyint(1) DEFAULT NULL,
              `style_dropdown` tinyint(1) DEFAULT '0',
              `style_default` varchar(255) DEFAULT NULL,
              `style_order` int(3) DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=InnoDB  DEFAULT CHARSET=utf8;"
        );

        $wpdb->query(
            "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}weblator_charts_style_dropdowns` (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `style_id` int(11) DEFAULT NULL,
              `dropdown_value` varchar(250) DEFAULT NULL,
              `dropdown_nicename` varchar(250) DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8;"
        );

        $id = $wpdb->get_var("SELECT id FROM {$wpdb->prefix}weblator_charts_style_options WHERE id = 1");
        //Insert Default Data

        if (!$id){

            $wpdb->query(
                "INSERT INTO `{$wpdb->prefix}weblator_charts_style_dropdowns` (`id`, `style_id`, `dropdown_value`, `dropdown_nicename`)
                VALUES
                    (1, 9, 'normal', 'normal'),
                    (2, 9, 'italic', 'italic'),
                    (3, 9, 'oblique', 'oblique'),
                    (4, 26, 'oblique', 'oblique'),
                    (5, 26, 'italic', 'italic'),
                    (6, 26, 'normal', 'normal'),
                    (7, 8, '8', '8px'),
                    (8, 8, '9', '9px'),
                    (9, 8, '10', '10px'),
                    (10, 8, '11', '11px'),
                    (11, 8, '12', '12px'),
                    (12, 8, '13', '13px'),
                    (13, 8, '14', '14px'),
                    (14, 8, '15', '15px'),
                    (15, 8, '16', '16px'),
                    (16, 8, '17', '17px'),
                    (17, 8, '18', '18px'),
                    (18, 8, '19', '19px'),
                    (19, 8, '20', '20px'),
                    (20, 8, '21', '21px'),
                    (21, 8, '22', '22px'),
                    (22, 27, '8', '8px'),
                    (23, 27, '9', '9px'),
                    (24, 27, '10', '10px'),
                    (25, 27, '11', '11px'),
                    (26, 27, '12', '12px'),
                    (27, 27, '13', '13px'),
                    (28, 27, '14', '14px'),
                    (29, 27, '15', '15px'),
                    (30, 27, '16', '16px'),
                    (31, 27, '17', '17px'),
                    (32, 27, '18', '18px'),
                    (33, 27, '19', '19px'),
                    (34, 27, '20', '20px'),
                    (35, 27, '21', '21px'),
                    (36, 27, '22', '22px');"
            );

            $wpdb->query(
                "INSERT INTO `{$wpdb->prefix}weblator_charts_style_options` (`id`, `style_option`, `style_key`, `style_label`, `style_description`, `style_colorpicker`, `style_bool`, `style_dropdown`, `style_default`, `style_order`)
                VALUES
                    (1, 'poll_fill_colour', NULL, 'Fill Colour', '', 1, 0, 0, 'rgba(151,187,205,0.5)', 1),
                    (2, 'poll_stroke_color', NULL, 'Stroke Colour', '', 1, 0, 0, 'rgba(151,187,205,1)', 2),
                    (3, 'poll_point_color', NULL, 'Point Colour', '', 1, 0, 0, 'rgba(151,187,205,1)', 3),
                    (4, 'poll_point_stroke_color', NULL, 'Point Stroke Colour', '', 1, 0, 0, 'rgba(255,255,255,1)', 4),
                    (5, 'poll_scale_line_color', 'scaleLineColor', 'Scale Line Colour', '', 1, 0, 0, 'rgba(0,0,0, .1)', 5),
                    (6, 'poll_scale_line_width', 'scaleLineWidth', 'Scale Line Width', 'Width of the scale line in pixels', 0, 0, 0, '1', 6),
                    (7, 'poll_scale_show_labels', 'scaleShowLabels', 'Show Scale Labels', '', 0, 1, 0, '1', 7),
                    (8, 'poll_scale_font_size', 'scaleFontSize', 'Scale Label Font Size', '', 0, 0, 1, '12', 8),
                    (9, 'poll_scale_font_style', 'scaleFontStyle', 'Scale Label Font Style', '', 0, 0, 1, 'normal', 9),
                    (10, 'poll_scale_font_colour', 'scaleFontColor', 'Scale Label Font Colour', '', 1, 0, 0, 'rgb(102, 102, 102)', 10),
                    (11, 'poll_scale_show_gridlines', 'scaleShowGridLines', 'Show Scale Grid Lines', 'Whether grid lines are shown across the chart', 0, 1, 0, '1', 11),
                    (12, 'poll_scale_gridline_color', 'scaleGridLineColor', 'Scale Grid Line Colour', '', 1, 0, 0, 'rgba(0,0,0, 0.1)', 12),
                    (13, 'poll_scale_gridline_width', 'scaleGridLineWidth', 'Scale Grid Line Width', 'Width of the grid lines in pixels', 0, 0, 0, '1', 13),
                    (14, 'poll_bar_show_stroke', 'barShowStroke', 'Show Bar Stroke', '', 0, 1, 0, '1', 18),
                    (15, 'poll_bar_stroke_width', 'barStrokeWidth', 'Bar Stroke Width', 'The width of the bar stroke in pixels', 0, 0, 0, '2', 19),
                    (16, 'poll_bar_value_spacing', 'barValueSpacing', 'Bar Value Spacing', 'Spacing between each of the X value sets in pixels', 0, 0, 0, '5', 20),
                    (17, 'poll_bar_dataset_spacing', 'barDatasetSpacing', 'Bar Dataset Spacing', 'Spacing between data sets within X values in pixels', 0, 0, 0, '1', 21),
                    (18, 'poll_scale_show_label_backdrop', 'scaleShowLabelBackdrop', 'Show Scale Label Backdrop', '', 0, 1, 0, '1', 14),
                    (19, 'poll_scale_backdrop_color', 'scaleBackdropColor', 'Scale Backdrop Colour', '', 1, 0, 0, 'rgba(255,255,255,0.75)', 15),
                    (20, 'poll_scale_backdrop_padding_y', 'scaleBackdropPaddingY', 'Scale Backdrop Padding Y', 'The backdrop padding above and below the label in pixels', 0, 0, 0, '2', 16),
                    (21, 'poll_scale_backdrop_padding_x', 'scaleBackdropPaddingX', 'Scale Backdrop Padding X', 'The backdrop padding to the side of the label in pixels', 0, 0, 0, '2', 17),
                    (23, 'poll_angle_show_line_out', 'angleShowLineOut', 'Show Angle Line Out', 'Whether we show the angle lines out of the radar', 0, 1, 0, '1', 22),
                    (24, 'poll_angle_line_color', 'angleLineColor', 'Angle Line Colour', 'Colour of the angle lines', 1, 0, 0, 'rgba(0,0,0,.1)', 23),
                    (25, 'poll_angle_line_width', 'angleLineWidth', 'Angle Line Width', 'Angle line width in pixels', 0, 0, 0, '1', 24),
                    (26, 'poll_point_label_font_style', 'pointLabelFontStyle', 'Point Label Font Style', '', 0, 0, 1, 'normal', 25),
                    (27, 'poll_point_font_size', 'pointLabelFontSize', 'Point Font Size', '', 0, 0, 1, '12', 26),
                    (28, 'poll_point_font_color', 'pointLabelFontColor', 'Point Font Colour', '', 1, 0, 0, 'rgb(102, 102, 102)', 27),
                    (29, 'poll_point_dot', 'pointDot', 'Point Dot', 'Whether to show a dot for each point', 0, 1, 0, '1', 28),
                    (30, 'poll_point_dot_radius', 'pointDotRadius', 'Point Dot Radius', 'Point dot radius in pixels', 0, 0, 0, '3', 29),
                    (31, 'poll_point_dot_stroke_width', 'pointDotStrokeWidth', 'Point Dot Stroke Width', 'Point dot stroke width in pixels', 0, 0, 0, '1', 30),
                    (32, 'poll_segment_show_state', 'segmentShowStroke', 'Segment Show Stroke', 'Stroke a line around each segment in the chart', 0, 1, 0, '1', 31),
                    (33, 'poll_segment_stroke_color', 'segmentStrokeColor', 'Segment Stroke Colour', 'The colour of the stroke around each segment', 1, 0, 0, 'rgba(255,255,255,1)', 32),
                    (34, 'poll_segment_stroke_width', 'segmentStrokeWidth', 'Segment Stroke Width', 'The width of the stroke in pixels', 0, 0, 0, '2', 33),
                    (35, 'bs_bar_striped', 'bsBarStriped', 'Striped Gradient', 'Apply striped pattern to progress bar', 0, 1, 0, '0', 34);"

            );

            $wpdb->query("
                INSERT INTO `{$wpdb->prefix}weblator_charts_style_chart_link` (`id`, `chart_id`, `style_id`)
                VALUES
                    (2, 6, 5),
                    (3, 5, 5),
                    (4, 4, 5),
                    (5, 1, 5),
                    (6, 6, 6),
                    (7, 5, 6),
                    (8, 4, 6),
                    (9, 1, 6),
                    (10, 6, 7),
                    (11, 5, 7),
                    (12, 4, 7),
                    (13, 1, 7),
                    (14, 6, 8),
                    (15, 5, 8),
                    (16, 4, 8),
                    (17, 1, 8),
                    (18, 6, 9),
                    (19, 5, 9),
                    (20, 4, 9),
                    (21, 1, 10),
                    (22, 6, 10),
                    (23, 5, 10),
                    (24, 4, 10),
                    (26, 6, 11),
                    (28, 4, 11),
                    (29, 1, 11),
                    (32, 4, 12),
                    (33, 1, 12),
                    (36, 4, 13),
                    (37, 1, 13),
                    (38, 1, 14),
                    (39, 1, 15),
                    (40, 1, 16),
                    (41, 1, 17),
                    (42, 6, 18),
                    (43, 6, 19),
                    (44, 6, 20),
                    (45, 6, 21),
                    (46, 5, 18),
                    (47, 5, 19),
                    (48, 5, 20),
                    (49, 5, 21),
                    (50, 5, 23),
                    (51, 5, 24),
                    (52, 5, 25),
                    (53, 5, 26),
                    (54, 5, 27),
                    (55, 5, 28),
                    (56, 5, 29),
                    (57, 5, 30),
                    (58, 5, 31),
                    (59, 3, 32),
                    (60, 3, 33),
                    (61, 3, 34),
                    (62, 2, 32),
                    (63, 2, 33),
                    (64, 2, 34),
                    (65, 6, 32),
                    (66, 6, 33),
                    (67, 6, 34),
                    (68, 1, 1),
                    (69, 1, 2),
                    (70, 4, 1),
                    (71, 4, 2),
                    (72, 4, 3),
                    (73, 4, 4),
                    (74, 5, 1),
                    (75, 5, 2),
                    (76, 5, 3),
                    (77, 5, 4),
                    (78, 7, 35),
                    (79, 1, 9);

            ");

            $wpdb->query(
                "INSERT INTO `{$wpdb->prefix}weblator_charts_chart_type` (`id`, `chart_type`, `order`)
                VALUES
                    (1, 'Bar Chart', 3),
                    (2, 'Pie Chart', 1),
                    (3, 'Doughnut Chart', 2),
                    (4, 'Line Chart', 4),
                    (5, 'Radar Chart', 6),
                    (6, 'Polar Chart', 5),
                    (7, 'Bootstrap Progress Bars', 7);
                "
            );

        }

        //1.2.0 update to add percentage values
        $check = $wpdb->get_results("SHOW COLUMNS FROM {$wpdb->prefix}weblator_charts_charts LIKE 'chart_percentage_values'");
        if (!count($check)){

            $wpdb->query("ALTER TABLE {$wpdb->prefix}weblator_charts_charts ADD COLUMN `chart_percentage_values` TINYINT(1) DEFAULT 0 AFTER `chart_legend_font_colour`");

        }

    }


    public function ajax_url(){

        if (get_bloginfo("version") < 3.3){

            $var = "<script>var ajaxurl = '" . admin_url('admin-ajax.php') . "'</script>";
            echo $var;

        }

    }

	/**
	 * Register and enqueue public-facing style sheet.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_slug . '-plugin-styles', plugins_url( 'assets/css/public.css', __FILE__ ), array(), self::VERSION );
	}

	/**
	 * Register and enqueues public-facing JavaScript files.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

        
        wp_enqueue_script( $this->plugin_slug . '-canvas', plugins_url( 'assets/js/excanvas.js', __FILE__ ), array( 'jquery' ), self::VERSION );
        wp_enqueue_script( $this->plugin_slug . '-charts', plugins_url( 'assets/js/chartjs_new.js', __FILE__ ), array( 'jquery', $this->plugin_slug . '-canvas' ), self::VERSION );
        wp_enqueue_script( $this->plugin_slug . '-legend', plugins_url( 'assets/js/legend.js', __FILE__ ), array( 'jquery' ), self::VERSION );
		wp_enqueue_script( $this->plugin_slug . '-plugin-script', plugins_url( 'assets/js/public.js', __FILE__ ), array( 'jquery', $this->plugin_slug . '-charts', $this->plugin_slug . '-canvas', $this->plugin_slug . '-legend' ), self::VERSION );
        wp_localize_script($this->plugin_slug . '-plugin-script', 'ajaxurl', admin_url('admin-ajax.php') );


    }


    public function display_chart($atts){

        global $wpdb;

        $var = null;

        if (isset($atts[0]) && $atts[0] == "random"){

            $random = $wpdb->get_var("SELECT id FROM " . WEBLATOR_CHARTS_PREFIX . "charts WHERE  chart_is_live = 1 AND deleted_date IS NULL ORDER BY RAND() LIMIT 1;");
            $chart    = $wpdb->get_row($wpdb->prepare("SELECT * FROM " . WEBLATOR_CHARTS_PREFIX . "charts WHERE id = %d ", $random), ARRAY_A);
            $options = $wpdb->get_results($wpdb->prepare("SELECT * FROM " . WEBLATOR_CHARTS_PREFIX . "chart_options WHERE chart_id = %d AND option_deleted_date IS NULL ORDER BY option_order ASC", $random), ARRAY_A);

        }else {

            $chart    = $wpdb->get_row($wpdb->prepare("SELECT * FROM " . WEBLATOR_CHARTS_PREFIX . "charts WHERE id = %d ", $atts["id"]), ARRAY_A);
            $options = $wpdb->get_results($wpdb->prepare("SELECT * FROM " . WEBLATOR_CHARTS_PREFIX . "chart_options WHERE chart_id = %d AND option_deleted_date IS NULL ORDER BY option_order ASC", $atts["id"]), ARRAY_A);

        }

        require("views/public.php");

        if ($chart["chart_is_live"])
            return $var;
        else
            return false;

    }

}
